@@ -28,7 +28,7 @@ from babel._compat import string_types, number_types, PY2, cmp
 __all__ = ['Message', 'Catalog', 'TranslationError']
 
 
-PYTHON_FORMAT = re.compile(r'''(?x)
+PYTHON_FORMAT = re.compile(r'''
     \%
         (?:\(([\w]*)\))?
         (
@@ -37,7 +37,39 @@ PYTHON_FORMAT = re.compile(r'''(?x)
             [hlL]?
         )
         ([diouxXeEfFgGcrs%])
-''')
+''', re.VERBOSE)
+
+
+def _parse_datetime_header(value):
+    match = re.match(r'^(?P<datetime>.*?)(?P<tzoffset>[+-]\d{4})?$', value)
+
+    tt = time.strptime(match.group('datetime'), '%Y-%m-%d %H:%M')
+    ts = time.mktime(tt)
+    dt = datetime.fromtimestamp(ts)
+
+    # Separate the offset into a sign component, hours, and # minutes
+    tzoffset = match.group('tzoffset')
+    if tzoffset is not None:
+        plus_minus_s, rest = tzoffset[0], tzoffset[1:]
+        hours_offset_s, mins_offset_s = rest[:2], rest[2:]
+
+        # Make them all integers
+        plus_minus = int(plus_minus_s + '1')
+        hours_offset = int(hours_offset_s)
+        mins_offset = int(mins_offset_s)
+
+        # Calculate net offset
+        net_mins_offset = hours_offset * 60
+        net_mins_offset += mins_offset
+        net_mins_offset *= plus_minus
+
+        # Create an offset object
+        tzoffset = FixedOffsetTimezone(net_mins_offset)
+
+        # Store the offset in a datetime object
+        dt = dt.replace(tzinfo=tzoffset)
+
+    return dt
 
 
 class Message(object):
@@ -51,7 +83,7 @@ class Message(object):
                    pluralizable messages
         :param string: the translated message string, or a
                        ``(singular, plural)`` tuple for pluralizable messages
-        :param locations: a sequence of ``(filenname, lineno)`` tuples
+        :param locations: a sequence of ``(filename, lineno)`` tuples
         :param flags: a set or sequence of flags
         :param auto_comments: a sequence of automatic comments for the message
         :param user_comments: a sequence of user comments for the message
@@ -61,10 +93,10 @@ class Message(object):
                        PO file, if any
         :param context: the message context
         """
-        self.id = id #: The message ID
+        self.id = id
         if not string and self.pluralizable:
             string = (u'', u'')
-        self.string = string #: The message translation
+        self.string = string
         self.locations = list(distinct(locations))
         self.flags = set(flags)
         if id and self.python_format:
@@ -242,15 +274,15 @@ class Catalog(object):
         :param charset: the encoding to use in the output (defaults to utf-8)
         :param fuzzy: the fuzzy bit on the catalog header
         """
-        self.domain = domain #: The message domain
+        self.domain = domain
         if locale:
             locale = Locale.parse(locale)
-        self.locale = locale #: The locale or `None`
+        self.locale = locale
         self._header_comment = header_comment
         self._messages = odict()
 
-        self.project = project or 'PROJECT' #: The project name
-        self.version = version or 'VERSION' #: The project version
+        self.project = project or 'PROJECT'
+        self.version = version or 'VERSION'
         self.copyright_holder = copyright_holder or 'ORGANIZATION'
         self.msgid_bugs_address = msgid_bugs_address or 'EMAIL@ADDRESS'
 
@@ -265,15 +297,15 @@ class Catalog(object):
             creation_date = datetime.now(LOCALTZ)
         elif isinstance(creation_date, datetime) and not creation_date.tzinfo:
             creation_date = creation_date.replace(tzinfo=LOCALTZ)
-        self.creation_date = creation_date #: Creation date of the template
+        self.creation_date = creation_date
         if revision_date is None:
             revision_date = 'YEAR-MO-DA HO:MI+ZONE'
         elif isinstance(revision_date, datetime) and not revision_date.tzinfo:
             revision_date = revision_date.replace(tzinfo=LOCALTZ)
-        self.revision_date = revision_date #: Last revision date of the catalog
-        self.fuzzy = fuzzy #: Catalog header fuzzy bit (`True` or `False`)
+        self.revision_date = revision_date
+        self.fuzzy = fuzzy
 
-        self.obsolete = odict() #: Dictionary of obsolete messages
+        self.obsolete = odict()  # Dictionary of obsolete messages
         self._num_plurals = None
         self._plural_expr = None
 
@@ -299,7 +331,7 @@ class Catalog(object):
 
     >>> catalog = Catalog(project='Foobar', version='1.0',
     ...                   copyright_holder='Foo Company')
-    >>> print catalog.header_comment #doctest: +ELLIPSIS
+    >>> print(catalog.header_comment) #doctest: +ELLIPSIS
     # Translations template for Foobar.
     # Copyright (C) ... Foo Company
     # This file is distributed under the same license as the Foobar project.
@@ -317,7 +349,7 @@ class Catalog(object):
     ... # This file is distributed under the same license as the PROJECT
     ... # project.
     ... #'''
-    >>> print catalog.header_comment
+    >>> print(catalog.header_comment)
     # The POT for my really cool Foobar project.
     # Copyright (C) 1990-2003 Foo Company
     # This file is distributed under the same license as the Foobar
@@ -342,10 +374,12 @@ class Catalog(object):
         else:
             headers.append(('PO-Revision-Date', self.revision_date))
         headers.append(('Last-Translator', self.last_translator))
+        if self.locale is not None:
+            headers.append(('Language', str(self.locale)))
         if (self.locale is not None) and ('LANGUAGE' in self.language_team):
             headers.append(('Language-Team',
-                           self.language_team.replace('LANGUAGE',
-                                                      str(self.locale))))
+                            self.language_team.replace('LANGUAGE',
+                                                       str(self.locale))))
         else:
             headers.append(('Language-Team', self.language_team))
         if self.locale is not None:
@@ -368,6 +402,8 @@ class Catalog(object):
                 self.msgid_bugs_address = value
             elif name == 'last-translator':
                 self.last_translator = value
+            elif name == 'language':
+                self.locale = Locale.parse(value)
             elif name == 'language-team':
                 self.language_team = value
             elif name == 'content-type':
@@ -379,63 +415,11 @@ class Catalog(object):
                 self._num_plurals = int(params.get('nplurals', 2))
                 self._plural_expr = params.get('plural', '(n != 1)')
             elif name == 'pot-creation-date':
-                # FIXME: this should use dates.parse_datetime as soon as that
-                #        is ready
-                value, tzoffset, _ = re.split('([+-]\d{4})$', value, 1)
-
-                tt = time.strptime(value, '%Y-%m-%d %H:%M')
-                ts = time.mktime(tt)
-
-                # Separate the offset into a sign component, hours, and minutes
-                plus_minus_s, rest = tzoffset[0], tzoffset[1:]
-                hours_offset_s, mins_offset_s = rest[:2], rest[2:]
-
-                # Make them all integers
-                plus_minus = int(plus_minus_s + '1')
-                hours_offset = int(hours_offset_s)
-                mins_offset = int(mins_offset_s)
-
-                # Calculate net offset
-                net_mins_offset = hours_offset * 60
-                net_mins_offset += mins_offset
-                net_mins_offset *= plus_minus
-
-                # Create an offset object
-                tzoffset = FixedOffsetTimezone(net_mins_offset)
-
-                # Store the offset in a datetime object
-                dt = datetime.fromtimestamp(ts)
-                self.creation_date = dt.replace(tzinfo=tzoffset)
+                self.creation_date = _parse_datetime_header(value)
             elif name == 'po-revision-date':
                 # Keep the value if it's not the default one
                 if 'YEAR' not in value:
-                    # FIXME: this should use dates.parse_datetime as soon as
-                    #        that is ready
-                    value, tzoffset, _ = re.split('([+-]\d{4})$', value, 1)
-                    tt = time.strptime(value, '%Y-%m-%d %H:%M')
-                    ts = time.mktime(tt)
-
-                    # Separate the offset into a sign component, hours, and
-                    # minutes
-                    plus_minus_s, rest = tzoffset[0], tzoffset[1:]
-                    hours_offset_s, mins_offset_s = rest[:2], rest[2:]
-
-                    # Make them all integers
-                    plus_minus = int(plus_minus_s + '1')
-                    hours_offset = int(hours_offset_s)
-                    mins_offset = int(mins_offset_s)
-
-                    # Calculate net offset
-                    net_mins_offset = hours_offset * 60
-                    net_mins_offset += mins_offset
-                    net_mins_offset *= plus_minus
-
-                    # Create an offset object
-                    tzoffset = FixedOffsetTimezone(net_mins_offset)
-
-                    # Store the offset in a datetime object
-                    dt = datetime.fromtimestamp(ts)
-                    self.revision_date = dt.replace(tzinfo=tzoffset)
+                    self.revision_date = _parse_datetime_header(value)
 
     mime_headers = property(_get_mime_headers, _set_mime_headers, doc="""\
     The MIME headers of the catalog, used for the special ``msgid ""`` entry.
@@ -451,7 +435,7 @@ class Catalog(object):
     >>> catalog = Catalog(project='Foobar', version='1.0',
     ...                   creation_date=created)
     >>> for name, value in catalog.mime_headers:
-    ...     print '%s: %s' % (name, value)
+    ...     print('%s: %s' % (name, value))
     Project-Id-Version: Foobar 1.0
     Report-Msgid-Bugs-To: EMAIL@ADDRESS
     POT-Creation-Date: 1990-04-01 15:30+0000
@@ -471,12 +455,13 @@ class Catalog(object):
     ...                   last_translator='John Doe <jd@example.com>',
     ...                   language_team='de_DE <de@example.com>')
     >>> for name, value in catalog.mime_headers:
-    ...     print '%s: %s' % (name, value)
+    ...     print('%s: %s' % (name, value))
     Project-Id-Version: Foobar 1.0
     Report-Msgid-Bugs-To: EMAIL@ADDRESS
     POT-Creation-Date: 1990-04-01 15:30+0000
     PO-Revision-Date: 1990-08-03 12:00+0000
     Last-Translator: John Doe <jd@example.com>
+    Language: de_DE
     Language-Team: de_DE <de@example.com>
     Plural-Forms: nplurals=2; plural=(n != 1)
     MIME-Version: 1.0
@@ -494,7 +479,7 @@ class Catalog(object):
         >>> Catalog(locale='en').num_plurals
         2
         >>> Catalog(locale='ga').num_plurals
-        3
+        5
 
         :type: `int`"""
         if self._num_plurals is None:
@@ -511,7 +496,7 @@ class Catalog(object):
         >>> Catalog(locale='en').plural_expr
         '(n != 1)'
         >>> Catalog(locale='ga').plural_expr
-        '(n==1 ? 0 : n==2 ? 1 : 2)'
+        '(n==1 ? 0 : n==2 ? 1 : n>=3 && n<=6 ? 2 : n>=7 && n<=10 ? 3 : 4)'
 
         :type: `string_types`"""
         if self._plural_expr is None:
@@ -642,7 +627,7 @@ class Catalog(object):
                    pluralizable messages
         :param string: the translated message string, or a
                        ``(singular, plural)`` tuple for pluralizable messages
-        :param locations: a sequence of ``(filenname, lineno)`` tuples
+        :param locations: a sequence of ``(filename, lineno)`` tuples
         :param flags: a set or sequence of flags
         :param auto_comments: a sequence of automatic comments
         :param user_comments: a sequence of user comments
@@ -690,7 +675,7 @@ class Catalog(object):
         if key in self._messages:
             del self._messages[key]
 
-    def update(self, template, no_fuzzy_matching=False):
+    def update(self, template, no_fuzzy_matching=False, update_header_comment=False):
         """Update the catalog based on the given template catalog.
 
         >>> from babel.messages import Catalog
@@ -737,7 +722,7 @@ class Catalog(object):
 
         >>> 'head' in catalog
         False
-        >>> catalog.obsolete.values()
+        >>> list(catalog.obsolete.values())
         [<Message 'head' (flags: [])>]
 
         :param template: the reference catalog, usually read from a POT file
@@ -796,7 +781,7 @@ class Catalog(object):
                     if no_fuzzy_matching is False:
                         # do some fuzzy matching with difflib
                         if isinstance(key, tuple):
-                            matchkey = key[0] # just the msgid, no context
+                            matchkey = key[0]  # just the msgid, no context
                         else:
                             matchkey = key
                         matches = get_close_matches(matchkey.lower().strip(),
@@ -814,6 +799,12 @@ class Catalog(object):
         for msgid in remaining:
             if no_fuzzy_matching or msgid not in fuzzy_matches:
                 self.obsolete[msgid] = remaining[msgid]
+
+        if update_header_comment:
+            # Allow the updated catalog's header to be rewritten based on the
+            # template's header
+            self.header_comment = template.header_comment
+
         # Make updated catalog's POT-Creation-Date equal to the template
         # used to update the catalog
         self.creation_date = template.creation_date
