@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 
+import contextlib
 import os
 import sys
 import shutil
@@ -12,9 +13,9 @@ except ImportError:
     from urllib import urlretrieve
 
 
-URL = 'http://unicode.org/Public/cldr/28/core.zip'
-FILENAME = 'core-28.zip'
-FILESUM = 'bc545b4c831e1987ea931b04094d7b9fc59ec3d8'
+URL = 'http://unicode.org/Public/cldr/29/core.zip'
+FILENAME = 'core-29.zip'
+FILESUM = '44d117e6e591a8f9655602ff0abdee105df3cabe'
 BLKSIZE = 131072
 
 
@@ -90,9 +91,8 @@ def main():
             shutil.rmtree(common_path)
 
         log('Extracting CLDR to \'%s\'', cldr_path)
-        z = zipfile.ZipFile(zip_path)
-        z.extractall(cldr_path)
-        z.close()
+        with contextlib.closing(zipfile.ZipFile(zip_path)) as z:
+            z.extractall(cldr_path)
 
     subprocess.check_call([
         sys.executable,
